==> hermes-5.6.0.0.rec2.tar.gz: hermes-5.6.0.0.rec2/app/controllers/hermes/api/dmo/users_controller.rb <==
# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des méthodes
class Hermes::Api::Dmo::UsersController < Hermes::Api::Dmo::ApplicationController
  require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $regex
  ].freeze

  ACCEPTABLE_PARAMS = %w[
   first_name
   last_name
   email
   gender
   phone_number
   auth_roles
  ].freeze

  REGEX_PARAMS = %w[
  ].freeze

  MUTATE_PARAMS = %i[
    first_name
    last_name
    email
    gender
    phone_number
    auth_roles
  ].freeze
        
  def mutate_params
    MUTATE_PARAMS
  end 

  NO_ACL = [
    "superadministrateur",
    "collaborateur offre logement",
    "cre",
    "rdsl",
    "entreprise",
    "bailleur",
  ]

  def current_user_use_acl 
    (NO_ACL & (current_user&.auth_roles || [])).empty?
  end 

  def initialize
    super
    @model = User
    @serializer = Hermes::UserSerializer
  end

  def show 
    return render_json_error(:not_found, :object_not_found) if !@instance || (current_user_use_acl && @instance.id != current_user.id)

    render(
      jsonapi: @instance,
      serializer: @serializer,
      **@options
    )
  end 

  def index
    acceptable_params *ACCEPTABLE_PARAMS
    regex_params *REGEX_PARAMS
    valid_operators *VALID_OPERATORS

    paginate(
      index_aggregation(no_acl: !current_user_use_acl),
      each_serializer: @serializer,
      **@options
    )
  end

  def check_for_beiam_error(user)
    user.instance_variable_get(:@create_user_response) && ![200, 201, 409].include?(user.instance_variable_get(:@create_user_response).code)
  end

  def beiam_error(user)
    {
      json: user.instance_variable_get(:@create_user_response).body,
      status: user.instance_variable_get(:@create_user_response).code
    }
  end
  

  def get_user_params(p, user = nil)
    {
      username: p[:email] || user&.login,
      first_name: p[:first_name] || user&.first_name,
      last_name: p[:last_name] || user&.last_name,
      email: p[:email] || user&.email,
      gender: p[:gender] || user&.gender || 'M',
      phone_number: p[:phone_number] || user&.phone_number,
      login: p[:email] || user&.login,
      auth_roles: p[:auth_roles] || user&.auth_roles,
      provider: HERMES_CONFIG['identity_provider_key'],
      create_user_data: {
        gender: p[:gender] || user&.gender || 'M',
        lastName: p[:last_name] || user&.last_name,
        firstName: p[:first_name] || user&.first_name,
        email: p[:email] || user&.email,
        login: p[:email] || user&.login,
        mobileNumber: p[:phone_number] || user&.phone_number,
        roles: p[:auth_roles] || user&.auth_roles,
      },
    }
  end

  def can_create_user
    [
      'Administrator',
      'SuperAdmin',
      'Manager',
      'superadministrateur',
      'entreprise',
      'bailleur',
      'rdsl',
      'cre',
    ].any? { |role| current_user_roles&.include?(role) }
  end

  def can_create_user_with_auth_roles(p)
    return true if !p[:auth_roles]
    restricted_roles = [
      'Administrator',
      'SuperAdmin',
      'Manager',
      'superadministrateur',
    ]
    restricted_roles.all? { |role| !p[:auth_roles]&.include?(role) } || restricted_roles.any? { |role| current_user_roles&.include?(role) }
  end


  def create
    return render_error_not_mutable(not_mutable) unless not_mutable.empty?
    return render_json_error(:unauthorized, :cannot_create_user) unless can_create_user

    p = mutate_valid_params

    return render_json_error(:unauthorized, :cannot_create_user_with_auth_roles) unless can_create_user_with_auth_roles(p)

    user = User.create(get_user_params(p).merge(password: Devise.friendly_token[0,20]))

    return render json: user.errors.messages, status: 400 if user.invalid?
    return render(beiam_error(user)) if check_for_beiam_error(user)

    render(jsonapi: user, serializer: @serializer, status: :created)
  end

  def set_instance
    @instance = @model.find_by(id: params[:id], **@query_precision) if params[:id]
  end

  def update
    return render_json_error(:not_found, :instance_not_found) unless @instance
    return render_error_not_mutable(not_mutable) unless not_mutable.empty?
    return render_json_error(:unauthorized, :cannot_create_user) unless can_create_user

    p = mutate_valid_params

    user_params = get_user_params(p, @instance)
    @instance.update(user_params)

    return render json: @instance.errors.messages, status: 400 unless @instance.save
    return render(beiam_error(@instance)) if check_for_beiam_error(@instance)

    render(jsonapi: @instance, serializer: @serializer, status: :ok)
  end

  def switch
    @target = User.find_by(id: params[:id])
    @target = User.find_by(main_applicant_id: params[:id]) || Hermes::Applicant.find_by(id: params[:id])&.user unless @target
    return render(json: { success: false, message: 'Utilisateur target non trouvé.' }, status: :not_found) unless @target
    return render nothing: true, status: :not_found unless @user

    if can_switch?
      if targeatable?
        success_switch_response
      else
        render json: {
          success: false,
          message: 'L\'utilisateur target doit être demandeur, bailleur ou entreprise.'
        }, status: :unprocessable_entity
      end
    else
      render json: {
        success: false,
        message: 'Vous n\'avez pas les droits nécessaire pour effectuer cette action.'
      }, status: :unauthorized
    end
  rescue ActionController::ParameterMissing => e
    render_json_error(:not_acceptable, :parameter_missing, { message: e.message })
  end

  private

  def can_switch?
    current_user_roles.include?('superadministrateur') ||
      current_user_roles.include?('collaborateur offre logement') ||
      current_user_roles.include?('cre')
  end

  def targeatable?
    return true if current_user_roles.include?('superadministrateur') || current_user_roles.include?('cre')

    (%w[demandeur bailleur entreprise] & @target.attributes.slice(:roles, :auth_roles).values.flatten.uniq.compact).any?
  end

  def success_switch_response
    token = @target.generate_jwt(true_user_id: @user.id.to_s)
    render json: {
      success: true,
      info: t(:registered),
      email: @target.email,
      login: @target.login,
      jwt_token: token
    }, status: :ok
  end
end

==> hermes-6.2.1.7.rec2.tar.gz: hermes-6.2.1.7.rec2/app/controllers/hermes/api/dmo/users_controller.rb <==
# frozen_string_literal: true

# checker Api::Dmo::HermesController avant d'ajouter des méthodes
class Hermes::Api::Dmo::UsersController < Hermes::Api::Dmo::ApplicationController
  require Rails.root.join('lib', 'core_extentions', 'parameters', 'handle_sort').to_s

  VALID_OPERATORS = %w[
    $in
    $nin
    $ne
    $gt
    $gte
    $lt
    $lte
    $regex
  ].freeze

  ACCEPTABLE_PARAMS = %w[
   first_name
   last_name
   email
   gender
   phone_number
   auth_roles
   inactive_at
   current_sign_in_at
   last_sign_in_at
  ].freeze

  SUBSCRIPTION_ACCEPTABLE_PARAMS = %w[
    keys_updated_at
   ].freeze

  REGEX_PARAMS = %w[
  ].freeze

  MUTATE_PARAMS = %i[
    first_name
    last_name
    email
    gender
    phone_number
    auth_roles
  ].freeze
        
  def mutate_params
    MUTATE_PARAMS
  end 

  NO_ACL = [
    "superadministrateur",
    "collaborateur offre logement",
    "cre",
    "rdsl",
    "entreprise",
    "bailleur",
  ]

  def current_user_use_acl 
    (NO_ACL & (current_user&.auth_roles || [])).empty?
  end 

  def can_see_all_users
    !(current_user_roles & ['Administrator', 'superadministrateur']).empty?
  end 

  def initialize
    super
    @model = User
    @serializer = Hermes::UserSerializer
  end

  def show 
    @instance = 
     if params[:id] == 'me'
        current_user
      elsif params[:id]
        @model.find_by(id: params[:id], **@query_precision)
      end 
      
    return render_json_error(:not_found, :object_not_found) if !@instance
    return render_json_error(:unauthorized, :cannot_see_users) if @instance.id != @user.id && !can_see_all_users

    render(
      jsonapi: @instance,
      serializer: @serializer,
      **@options
    )
  end 

  def index
    return render_json_error(:unauthorized, :cannot_see_users) unless can_see_all_users

    acceptable_params *ACCEPTABLE_PARAMS
    regex_params *REGEX_PARAMS
    valid_operators *VALID_OPERATORS

    paginate(
      index_aggregation(no_acl: !current_user_use_acl),
      each_serializer: @serializer,
      **@options
    )
  end

  def switch
    @target = User.find_by(id: params[:id])
    @target = User.find_by(main_applicant_id: params[:id]) || Hermes::Applicant.find_by(id: params[:id])&.user unless @target
    return render(json: { success: false, message: 'Utilisateur target non trouvé.' }, status: :not_found) unless @target
    return render nothing: true, status: :not_found unless @user

    if can_switch?
      if targeatable?
        success_switch_response
      else
        render json: {
          success: false,
          message: 'L\'utilisateur target doit être demandeur, bailleur ou entreprise.'
        }, status: :unprocessable_entity
      end
    else
      render json: {
        success: false,
        message: 'Vous n\'avez pas les droits nécessaire pour effectuer cette action.'
      }, status: :unauthorized
    end
  rescue ActionController::ParameterMissing => e
    render_json_error(:not_acceptable, :parameter_missing, { message: e.message })
  end

  def invalidate_token
    return render_json_error(:unprocessable_entity, :missing_params) if params[:token].blank?
    return render_json_error(:not_found, :token_not_found) unless user_token = Hermes::UserToken.find_by(user_id: current_user.id, token: params[:token])
    return render(json: user_token.errors, status: :unprocessable_entity) unless user_token.destroy

    render(body: nil, status: :no_content)
  end 

  private

  def can_switch?
    current_user_roles.include?('superadministrateur') ||
      current_user_roles.include?('collaborateur offre logement') ||
      current_user_roles.include?('cre')
  end

  def targeatable?
    return true if current_user_roles.include?('superadministrateur') || current_user_roles.include?('cre')

    (%w[demandeur bailleur entreprise] & @target.attributes.slice(:roles, :auth_roles).values.flatten.uniq.compact).any?
  end

  def success_switch_response
    token = @target.generate_jwt(true_user_id: @user.id.to_s)
    render json: {
      success: true,
      info: t(:registered),
      email: @target.email,
      login: @target.login,
      jwt_token: token
    }, status: :ok
  end
end
